%% demo_SUnGP [1]
%This is a short demo for [1].
%   [1] Naveed Akhtar, Faisal Shafait and Ajmal Mian,'SUnGP: Agreedy sparse
%   approximation algorithm for hyperspectral unmixing', In Proc. ICPR,2014.
%
%%----------To run-------------------------------------------------
% Add the folder to Matlab path and run this file
%
%%----------Description--------------------------------------------
% The implementation provides approximate solution to the following problem:
% min_a ||Da - y||_2 s.t. ||a||_0 =< p & a >= 0.
% In the above:
%       - D in R^(m x k): dictionary/spectral library (remains unchanged)
%       - a in R^(k x 1): sparse code vector/fractional abundances
%       - y in R^(m x 1): input signal/mixed hyperspectral pixel
%
% Author: Naveed Akhtar, 2014
%__________________________________________________________________________

%%-----------------Parameter Settings ------------------------
pixels = 500;           % Number of pixels in the synthetic image
p = 5;                  % Cardinality or sparsity of the mixed pixel
c = 2;                  % band difference for the derivative
SNR = 50;               % SNR in dBs.


%Algorithm parameters
param.desiredIteraitons = p;        %maximum iterations for criterion (a)
param.L = 50;                       %atoms selected before pruning in each iteration

%%---------------Image creation--------------------------------
d = load('dictionary');
D = d.D;
[m, k] = size(D);

[Y,A, Aval] = createImage(D, p, pixels);
Y = addnoise(Y,SNR);
D = D./repmat(sum(D),m,1);


fprintf('\nImage specifications... ')
disp(['Pixels: ' num2str(pixels)])
disp(['SNR: ' num2str(SNR)])
disp(['Sparsity/Cardinality: ' num2str(p)])

%%---------------Data processing -----------------------------
Yd = specDerivative(Y,c);
Dd = specDerivative(D,c);
fprintf ('\nSparse coding...')
tic
A_SUnGP = SUnGP(Dd,Yd,param);
t = toc;
disp (['Total time for sparse approximation: ' num2str(t) ' seconds.'])

disp ('Processing for evaluation of the learned sparse codes ....')
ASUnGP = [];
erecovery_SUnGP = 0;
UESUnGP = [];
alphaNorm = [];
for i = 1:size(Y, 2)
     eSUnGP = supportAnalysis(A_SUnGP(:,i), A(:,i));
     erecovery_SUnGP = erecovery_SUnGP + eSUnGP;
     [ue,a_SUnGP_val]  = myAnalyzeUnmixErr(A_SUnGP(:,i), Aval(:,i), D, Y(:,i));
     UESUnGP = [UESUnGP ue];
end
SUnGP_Recovery = erecovery_SUnGP/size(Y, 2);
EuDistance = sum(UESUnGP,2)/length(UESUnGP);
UF_SUnGP = (SUnGP_Recovery(1,1)/p);
disp (['Mean Unmixing Fidelity: ' num2str(UF_SUnGP)])
disp (['Mean Euclidean distance: ' num2str(EuDistance)])
